/********************************************************************/
/*                                                                  */
/*   Program Name :  datachk                                        */
/*                                                                  */
/*      Version:  0.0   2001-08-17                                  */
/*                2.10  2002-06-04                                  */
/*                2.11  2002-10-21  to allow extended file          */
/*                2.20  2004-08-12  add sampling statistics         */
/*                                                                  */
/*                                                                  */
/*      Data Check for PC-VSSP(IP-VLBI) board                       */
/*                                                                  */
/*                                                                  */
/*      Copyright (c) 2002-2004 T.Kondo/NICT All Right Reserved     */
/*                                                                  */
/********************************************************************/
/*
       gp@  datachk file_name [mode]
                   file_name -- f[^t@C
                          mode     --  1: TvOf[^v\
                          @@@@@@ 0: \ȂiftHgj@

        ver 2.11  g[hō쐬ꂽt@CI
                  ǂݍł悤ɉC
        ver 2004-081-12  TvOf[^́{̊ŕ\@\ǉ
*/
#define DATACHK_VERSION "Ver. 2004-08-12r"

#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi.h"  // IP-VLBIp֐Q(by T.Kondo)


void chkfilen(char* fname)
{
	   unsigned char bit64[8];
       unsigned char ibuf[5000];
	   int nbyte=5000;
       int i,iok,spatn,smplbit,numch,hh,mm,ss;
	   long int seconds,numb,npos, secold, tflag, nkai;
       long int bitsec,bitint;
	   int sfreq;
       FILE *stream;
       int bytesread, rbytes;
       int f_c;
       char cbuf[10], fname2[80];
       double dbytes;

	   numb=0;
	   npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;

       printf("FILE : %s\n",fname);

       if( (stream  = fopen( fname, "rb" )) == NULL ) {
           printf("File Open Error! \n");
		   goto exitend2;
       }	

       printf("FMT A/D CH  f(kHz)  TIME   seconds   # of bits\n");

       bytesread = fread( ibuf, sizeof( char ), nbyte, stream );
       dbytes+=(double)bytesread;
 
       if( ferror( stream ) || bytesread==0 ) {
           printf("File EOF or Read Error!\n");
		   goto exitend;
       }
       for (i=0; i <8 ; i++){
		  bit64[i]=ibuf[i];
       }

       rbytes=bytesread;

loop1:

       headerchkn(bit64,&iok,&spatn,&smplbit,&sfreq,&numch,&seconds,      
		         &hh,&mm,&ss);
       if (iok == 0) {  // sync detection
           if (numb >= 8 ){
              bitint=(numb-8)*8;
              bitsec=sfreq*1000*smplbit*numch;
              printf("  %d",bitint);
              if (bitint != bitsec){
                  printf("NG "); 
                  if(tflag ==0){
                     printf("Warning! (# of bits Error)\n"); 
                  } else {
                     printf("Warning! (TIME DISCON and Bits error!)\n");
                  }

               } else {
                  if(tflag==0){
                     printf("  \n"); 
                  } else {
                     printf(" Warning! (TIME DISCON)\n");
                  }
               }
           }
           if(nkai >0 && seconds-secold !=1){
                printf("New %2d  %2d   %4d %02d:%02d:%02d %6dNG", 
                      smplbit,numch, sfreq,hh,mm,ss,seconds);
                tflag=1;
           } else {
                printf("New %2d  %2d   %4d %02d:%02d:%02d %6d  ", 
                      smplbit,numch, sfreq,hh,mm,ss,seconds);
                tflag=0;
           }
           secold=seconds;
           //nkai++;
           numb=0;
       }
       numb++;
       npos++;
       if (npos >= rbytes){
		    npos=0;
            rbytes=bytesread;
       }

       for (i=0; i <7 ; i++){
		    bit64[i]=bit64[i+1];
       }
     
       if (npos+7 == rbytes){  // next record read
            //printf("Next Record Read!\n"); //debug
            bytesread = fread( ibuf, sizeof( char ), nbyte, stream );
            dbytes+=(double)bytesread;
            if (bytesread ==0 ) {// EOF
	            i=fclose( stream ) ;
			    f_c++;
			    // continuous file name creation     XXXX.datN   where N=1,2,3......
			    strcpy(fname2,fname);
			    sprintf(cbuf,"%d\0",f_c);
			    strcat(fname2,cbuf);
                /* file open */
                if( (stream  = fopen( fname2, "rb" )) == NULL ){
                    // file not found
		            goto exitend2;
                }	
			    printf("\n continuous file (%s) opened!\n",fname2);  
                printf("                                   ");  
                bytesread = fread( ibuf, sizeof( char ), nbyte, stream );
                dbytes+=(double)bytesread;
            }
            if( ferror( stream )  ) {
			    printf("File Read Error!\n");
			    goto exitend;
            }
       }
       if (npos+7 < rbytes ){
            bit64[7]=ibuf[npos+7];
       } else {
            bit64[7]=ibuf[npos+7-rbytes];
       }
       goto loop1;
	
 exitend:
	    i=fclose( stream ) ;
        printf("\n");
 exitend2:;
        printf("\n");
        printf("%f bytes read\n",dbytes); 
}


void chkfilen2(char* fname)
/* check with sampling statistics 2004-08-12 */
{
	   unsigned char bit64[8];
       unsigned char ibuf[5000];
	   int nbyte=5000;
       int xdata[40000];  // 40000=8*5000
       int xhist[8];     // 8=4ch*2  (" means sign+  sign- of data)
       int xhalf;
       int i,iok,spatn,smplbit,numch,hh,mm,ss;
       int n,k;
	   long int seconds,numb,npos, secold, tflag, nkai;
       long int bitsec,bitint;
	   int sfreq;
       FILE *stream;
       int bytesread, rbytes;
       int pos;
       int f_c;
       char cbuf[10], fname2[80];
       double dbytes;

	   numb=0;
	   npos=0;
       nkai=0;

       tflag=0;
       f_c=0;
       dbytes=0.0;

       for(i=0; i<8; i++) xhist[i]=0; 

       printf("FILE : %s\n",fname);

       if( (stream  = fopen( fname, "rb" )) == NULL ) {
           printf("File Open Error! \n");
		   goto exitend2;
       }	

       printf("                                                    %% of plus sign data  \n");
       printf("FMT A/D CH  f(kHz)  TIME   seconds    # of bits    Ch1    Ch2    Ch3    Ch4\n");

       bytesread = fread( ibuf, sizeof( char ), nbyte, stream );
       dbytes+=(double)bytesread;
 
       if( ferror( stream ) || bytesread==0 ) {
           printf("File EOF or Read Error!\n");
		   goto exitend;
       }
       for (i=0; i <8 ; i++){
		  bit64[i]=ibuf[i];
       }

       rbytes=bytesread;

loop1:

       headerchkn(bit64,&iok,&spatn,&smplbit,&sfreq,&numch,&seconds,      
		         &hh,&mm,&ss);
       xhalf=(int)(pow(2,smplbit-1)+0.5);
       if (iok == 0) {  // sync detection
           if (numb >= 8 ){
              bitint=(numb-8)*8;
              bitsec=sfreq*1000*smplbit*numch;
              if(numch==1){
                 printf("%12d %6.1f",bitint,
                  100.0*(double)xhist[1]/(double)(xhist[0]+xhist[1]));
              } else {
                 printf("%12d %6.1f %6.1f %6.1f %6.1f",bitint,
                  100.0*(double)xhist[1]/(double)(xhist[0]+xhist[1]),
                  100.0*(double)xhist[1*2+1]/(double)(xhist[1*2+0]+xhist[1*2+1]),
                  100.0*(double)xhist[2*2+1]/(double)(xhist[2*2+0]+xhist[2*2+1]),
                  100.0*(double)xhist[3*2+1]/(double)(xhist[3*2+0]+xhist[3*2+1]));
              }
              if (bitint != bitsec){
                  printf("NG "); 
                  if(tflag ==0){
                     printf("Warning! (# of bits Error)\n"); 
                  } else {
                     printf("Warning! (TIME DISCON and Bits error!)\n");
                  }

               } else {
                  if(tflag==0){
                     printf("  \n"); 
                  } else {
                     printf(" Warning! (TIME DISCON)\n");
                  }
               }
           }
           if(nkai >0 && seconds-secold !=1){
                printf("New %2d  %2d   %4d %02d:%02d:%02d %6dNG", 
                      smplbit,numch, sfreq,hh,mm,ss,seconds);
                tflag=1;
           } else {
                printf("New %2d  %2d   %4d %02d:%02d:%02d %6d  ", 
                      smplbit,numch, sfreq,hh,mm,ss,seconds);
                tflag=0;
           }
           for(i=0; i<8; i++) xhist[i]=0; 
           secold=seconds;
           //nkai++;
           numb=0;
       }
       numb++;
       npos++;
       if (npos >= rbytes){
		    npos=0;
            rbytes=bytesread;
       }

       for (i=0; i <7 ; i++){
		    bit64[i]=bit64[i+1];
       }
     
       if (npos+7 == rbytes){  // next record read
            //printf("Next Record Read!\n"); //debug
            bytesread = fread( ibuf, sizeof( char ), nbyte, stream );
            parseADdata_3(ibuf, bytesread, smplbit, xdata, &pos);
            dbytes+=(double)bytesread;

            // sampling balance statistics (only check +/- unbalance)
            if(numch>0){
               for(n=0; n<numch; n++){
                   for(k=0; k<pos/numch; k++){
                       if(xdata[k*numch+n] >= xhalf){
                           xhist[n*2+1]+=1;
                       } else {
                           xhist[n*2]+=1;
                       }
                   }
               }
            }

            if (bytesread ==0 ) {// EOF
	            i=fclose( stream ) ;
			    f_c++;
			    // continuous file name creation     XXXX.datN   where N=1,2,3......
			    strcpy(fname2,fname);
			    sprintf(cbuf,"%d\0",f_c);
			    strcat(fname2,cbuf);
                /* file open */
                if( (stream  = fopen( fname2, "rb" )) == NULL ){
                    // file not found
		            goto exitend2;
                }	
			    printf("\n continuous file (%s) opened!\n",fname2);  
                printf("                                   ");  
                bytesread = fread( ibuf, sizeof( char ), nbyte, stream );
                parseADdata_3(ibuf, bytesread, smplbit, xdata, &pos);
                dbytes+=(double)bytesread;
                // sampling balance statistics (only check +/- unbalance)
                if(numch>0){
                   for(n=0; n<numch; n++){
                       for(k=0; k<pos/numch; k++){
                           if(xdata[k*numch+n] >= xhalf){
                               xhist[n*2+1]+=1;
                           } else {
                               xhist[n*2]+=1;
                           }
                       }
                   }
                }
            }
            if( ferror( stream )  ) {
			    printf("File Read Error!\n");
			    goto exitend;
            }
       }
       if (npos+7 < rbytes ){
            bit64[7]=ibuf[npos+7];
       } else {
            bit64[7]=ibuf[npos+7-rbytes];
       }
       goto loop1;
	
 exitend:
	    i=fclose( stream ) ;
        printf("\n");
 exitend2:;
        printf("\n");
        printf("%f bytes read\n",dbytes); 
}



int main(int argc, char *argv[])
{
    char fname[60];
	char* dflt_fname="tds.data";
    int mode;
    mode=0;
    if(argc < 2) {
       //strcpy(fname,dflt_fname);
       printf("datachk %s\n",DATACHK_VERSION);
       printf("How to run: \n");
       printf("  datachk datafile [mode]\n");
       printf("\n");
       printf("    where  datafile ----- K5 data file name to check\n");
       //printf("                          if 0 is given, internal dflt name will be used\n");
       printf("           mode      ---  display sampling statistics (zero balance)\n");
       printf("                           1: ON   0: no display (default)\n");
       exit(0);
	}
    if (argc>=2){
       strcpy(fname,*(argv+1));
    } 
    if(argc >=3){
       mode = atoi(*(argv + 2 ));
    }
    printf("**** datachk  %s ****\n",DATACHK_VERSION);
    if(mode==1){
       chkfilen2(fname);
    } else {
       chkfilen(fname);
    }
	return 0;
}
